package com.sitemesh.parser;

import com.sitemesh.Page;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import org.openxml.parser.HTMLParser;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Implementation of {@link com.sitemesh.PageParser} that builds up a
 * {@link com.sitemesh.HTMLPage} using DOM.
 *
 * <p>This implementation uses <a href="http://www.openxml.org/">OpenXML 1.2</a>
 * as it is capable of parsing (and beform basic clean-up operations) on
 * HTML.</p>
 *
 * @author <a href="joe@truemesh.com">Joe Walnes</a>
 * @version $Revision: 1.7 $
 */
public class DOMPageParser extends AbstractPageParser {
	/**
	 * @label builds
	 * @directed
	 */
	/*#private DOMPage lnkDOMPage;*/

	/**
	 * Send data to DOM parser and build new
	 * {@link com.sitemesh.parser.DOMPage}. If the data
	 * cannot be parsed, an {@link com.sitemesh.parser.UnParsedPage}
	 * is returned instead.
	 */
	public Page parse( byte[] data ) throws IOException {
		try {
			HTMLDocument document = parseHTML( data );
			return new DOMPage( data, document );
		}
		catch ( SAXException e ) {
			return new UnParsedPage( data );
		}
	}

	/**
	 * Parse the data and return a DOM HTMLDocument.
	 */
	private HTMLDocument parseHTML( byte[] data ) throws IOException, SAXException {
		HTMLParser parser = new HTMLParser();
		parser.parse(
			new InputSource(
			new ByteArrayInputStream( data ) ) );
		return ( HTMLDocument )parser.getDocument();
	}

}
